@@ -5,8 +5,6 @@
  *      Author: utsugi
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <stdbool.h>
 #include <math.h>
 
@@ -19,12 +17,8 @@ penalty_ssmooth (const int n)
 {
 	int		i, j, k;
 	mm_sparse	*s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, n - 1, n, 2 * (n - 1));
-	s->i = (int *) malloc (s->nz * sizeof (int));
-	s->p = (int *) malloc ((s->n + 1) * sizeof (int));
-	s->data = (double *) malloc (s->nz * sizeof (double));
 
 	k = 0;
-	s->p[0] = 0;
 	for (j = 0; j < n; j++) {
 		if (j > 0) {
 			s->i[k] = j - 1;
@@ -44,7 +38,6 @@ penalty_dsmooth (const int n)
 {
 	int		j;
 	mm_dense	*d = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, n - 1, n, (n - 1) * n);
-	d->data = (double *) malloc (d->nz * sizeof (double));
 	mm_real_set_all (d, 0.);
 	for (j = 0; j < n; j++) {
 		if (j > 0) d->data[j - 1 + j * d->m] = -1.;
