@@ -11,12 +11,12 @@
 
 #include <mmreal.h>
 
-#include "private/atomic.h"
 #include "private/private.h"
+#include "private/atomic.h"
 
-/* mm_real supports real symmetric or general sparse, and real general dense matrix */
+/* mm_real supports real symmetric/general sparse/dense matrix */
 static bool
-is_type_supported (MM_typecode typecode)
+is_type_supported (const MM_typecode typecode)
 {
 	// invalid type
 	if (!mm_is_valid (typecode)) return false;
@@ -35,13 +35,13 @@ is_type_supported (MM_typecode typecode)
 
 /* check format */
 static bool
-is_format_valid (MMRealFormat format) {
+is_format_valid (const MMRealFormat format) {
 	return (format == MM_REAL_SPARSE || format == MM_REAL_DENSE);
 }
 
 /* check symmetric */
 static bool
-is_symm_valid (MMRealSymm symm)
+is_symm_valid (const MMRealSymm symm)
 {
 	return (symm == MM_REAL_GENERAL || symm == MM_REAL_SYMMETRIC_UPPER
 			|| symm == MM_REAL_SYMMETRIC_LOWER);
@@ -51,36 +51,38 @@ is_symm_valid (MMRealSymm symm)
 static mm_real *
 mm_real_alloc (void)
 {
-	mm_real	*mm = (mm_real *) malloc (sizeof (mm_real));
-	mm->m = 0;
-	mm->n = 0;
-	mm->nz = 0;
-	mm->i = NULL;
-	mm->p = NULL;
-	mm->data = NULL;
+	mm_real	*x = (mm_real *) malloc (sizeof (mm_real));
+	if (x == NULL) return NULL;
 
-	mm->symm = MM_REAL_GENERAL;
+	x->m = 0;
+	x->n = 0;
+	x->nnz = 0;
+	x->i = NULL;
+	x->p = NULL;
+	x->data = NULL;
+
+	x->symm = MM_REAL_GENERAL;
 
 	/* set typecode = "M_RG" : Matrix Real General */
 	// typecode[3] = 'G' : General
-	mm_initialize_typecode (&mm->typecode);
+	mm_initialize_typecode (&x->typecode);
 	// typecode[0] = 'M' : Matrix
-	mm_set_matrix (&mm->typecode);
+	mm_set_matrix (&x->typecode);
 	// typecode[2] = 'R' : Real
-	mm_set_real (&mm->typecode);
+	mm_set_real (&x->typecode);
 
-	return mm;
+	return x;
 }
 
 /*** create new mm_real object
  * MMRealFormat	format: MM_REAL_DENSE or MM_REAL_SPARSE
  * MMRealSymm		symm  : MM_REAL_GENERAL, MM_REAL_SYMMETRIC_UPPER or MM_REAL_SYMMETRIC_LOWER
  * int				m, n  : rows and columns of the matrix
- * int				nz    : number of nonzero elements of the matrix ***/
+ * int				nnz   : number of nonzero elements of the matrix ***/
 mm_real *
-mm_real_new (MMRealFormat format, MMRealSymm symm, const int m, const int n, const int nz)
+mm_real_new (MMRealFormat format, MMRealSymm symm, const int m, const int n, const int nnz)
 {
-	mm_real	*mm;
+	mm_real	*x;
 	bool		symmetric;
 
 	if (!is_format_valid (format))
@@ -92,59 +94,76 @@ mm_real_new (MMRealFormat format, MMRealSymm symm, const int m, const int n, con
 	if (symmetric && m != n)
 		error_and_exit ("mm_real_new", "symmetric matrix must be square.", __FILE__, __LINE__);
 
-	mm = mm_real_alloc ();
-	mm->m = m;
-	mm->n = n;
-	mm->nz = nz;
+	x = mm_real_alloc ();
+	if (x == NULL) error_and_exit ("mm_real_new", "failed to allocate object.", __FILE__, __LINE__);
+	x->m = m;
+	x->n = n;
+	x->nnz = nnz;
 
 	// typecode[1] = 'C' or 'A'
-	if (format == MM_REAL_SPARSE) mm_set_coordinate (&mm->typecode);
-	else mm_set_array (&mm->typecode);
+	if (format == MM_REAL_SPARSE) {
+		mm_set_coordinate (&x->typecode);
+		x->i = (int *) malloc (x->nnz * sizeof (int));
+		x->p = (int *) malloc ((x->n + 1) * sizeof (int));
+		if (x->i == NULL || x->p == NULL) error_and_exit ("mm_real_new", "cannot allocate memory.", __FILE__, __LINE__);
+	} else mm_set_array (&x->typecode);
 
-	mm->symm = symm;
+	x->symm = symm;
 	// typecode[3] = 'S'
-	if (symmetric) mm_set_symmetric (&mm->typecode);
+	if (symmetric) mm_set_symmetric (&x->typecode);
 
-	if (!is_type_supported (mm->typecode)) {
+	if (!is_type_supported (x->typecode)) {
 		char	msg[128];
-		sprintf (msg, "matrix type does not supported :[%s].", mm_typecode_to_str (mm->typecode));
+		sprintf (msg, "matrix type does not supported :[%s].", mm_typecode_to_str (x->typecode));
 		error_and_exit ("mm_real_new", msg, __FILE__, __LINE__);
 	}
+	if (format == MM_REAL_SPARSE) mm_set_coordinate (&x->typecode);
 
-	return mm;
+	// allocate arrays
+	x->data = (double *) malloc (x->nnz * sizeof (double));
+	if (x->data == NULL) error_and_exit ("mm_real_new", "cannot allocate memory.", __FILE__, __LINE__);
+	if (mm_real_is_sparse (x)) {
+		x->i = (int *) malloc (x->nnz * sizeof (int));
+		x->p = (int *) malloc ((x->n + 1) * sizeof (int));
+		if (x->i == NULL || x->p == NULL) error_and_exit ("mm_real_new", "cannot allocate memory.", __FILE__, __LINE__);
+		// initialize x->p[0]
+		x->p[0] = 0;
+	}
+
+	return x;
 }
 
 /*** free mm_real ***/
 void
-mm_real_free (mm_real *mm)
+mm_real_free (mm_real *x)
 {
-	if (mm) {
-		if (mm_real_is_sparse (mm)) {
-			if (mm->i) free (mm->i);
-			if (mm->p) free (mm->p);
+	if (x) {
+		if (mm_real_is_sparse (x)) {
+			if (x->i) free (x->i);
+			if (x->p) free (x->p);
 		}
-		if (mm->data) free (mm->data);
-		free (mm);
+		if (x->data) free (x->data);
+		free (x);
 	}
 	return;
 }
 
 /*** reallocate mm_real ***/
 bool
-mm_real_realloc (mm_real *mm, const int nz)
+mm_real_realloc (mm_real *x, const int nnz)
 {
-	if (mm->nz == nz) return true;
-	mm->data = (double *) realloc (mm->data, nz * sizeof (double));
-	if (mm->data == NULL) return false;
-	if (mm_real_is_sparse (mm)) {
-		mm->i = (int *) realloc (mm->i, nz * sizeof (int));
-		if (mm->i == NULL) return false;
+	if (x->nnz == nnz) return true;
+	x->data = (double *) realloc (x->data, nnz * sizeof (double));
+	if (x->data == NULL) return false;
+	if (mm_real_is_sparse (x)) {
+		x->i = (int *) realloc (x->i, nnz * sizeof (int));
+		if (x->i == NULL) return false;
 	}
-	mm->nz = nz;
+	x->nnz = nnz;
 	return true;
 }
 
-/*** set sparse ***/
+/*** set to sparse ***/
 void
 mm_real_set_sparse (mm_real *x)
 {
@@ -153,7 +172,7 @@ mm_real_set_sparse (mm_real *x)
 	return;
 }
 
-/*** set dense ***/
+/*** set to dense ***/
 void
 mm_real_set_dense (mm_real *x)
 {
@@ -162,7 +181,7 @@ mm_real_set_dense (mm_real *x)
 	return;
 }
 
-/*** set general ***/
+/*** set to general ***/
 void
 mm_real_set_general (mm_real *x)
 {
@@ -172,36 +191,36 @@ mm_real_set_general (mm_real *x)
 	return;
 }
 
-/*** set symmetric
+/*** set to symmetric
  * by default, assume symmetric upper
  * i.e., x->symm is set to MM_SYMMETRIC | MM_UPPER ***/
 void
 mm_real_set_symmetric (mm_real *x)
 {
-	if (mm_real_is_symmetric (x)) return;
 	if (x->m != x->n) error_and_exit ("mm_real_set_symmetric", "symmetric matrix must be square.", __FILE__, __LINE__);
+	if (mm_real_is_symmetric (x)) return;
 	mm_set_symmetric (&(x->typecode));
 	x->symm = MM_SYMMETRIC | MM_UPPER;	// by default, assume symmetric upper
 	return;
 }
 
-/*** set upper ***/
+/*** set to symmetric upper ***/
 void
 mm_real_set_upper (mm_real *x)
 {
-	if (!mm_real_is_symmetric (x)) error_and_exit ("mm_real_set_upper", "matrix must be symmetric.", __FILE__, __LINE__);
 	if (x->m != x->n) error_and_exit ("mm_real_set_upper", "symmetric matrix must be square.", __FILE__, __LINE__);
+	if (!mm_real_is_symmetric (x)) error_and_exit ("mm_real_set_upper", "matrix must be symmetric.", __FILE__, __LINE__);
 	if (mm_real_is_upper (x)) return;
 	x->symm = MM_SYMMETRIC | MM_UPPER;
 	return;
 }
 
-/*** set lower ***/
+/*** set to symmetric lower ***/
 void
 mm_real_set_lower (mm_real *x)
 {
-	if (!mm_real_is_symmetric (x)) error_and_exit ("mm_real_set_lower", "matrix must be symmetric.", __FILE__, __LINE__);
 	if (x->m != x->n) error_and_exit ("mm_real_set_lower", "symmetric matrix must be square.", __FILE__, __LINE__);
+	if (!mm_real_is_symmetric (x)) error_and_exit ("mm_real_set_lower", "matrix must be symmetric.", __FILE__, __LINE__);
 	if (mm_real_is_lower (x)) return;
 	x->symm = MM_SYMMETRIC | MM_LOWER;
 	return;
@@ -212,17 +231,11 @@ static mm_sparse *
 mm_real_copy_sparse (const mm_sparse *src)
 {
 	int			k;
-	mm_sparse	*dest = mm_real_new (MM_REAL_SPARSE, src->symm, src->m, src->n, src->nz);
+	mm_sparse	*dest = mm_real_new (MM_REAL_SPARSE, src->symm, src->m, src->n, src->nnz);
 
-	dest->i = (int *) malloc (src->nz * sizeof (int));
-	dest->p = (int *) malloc ((src->n + 1) * sizeof (int));
-	dest->data = (double *) malloc (src->nz * sizeof (double));
-
-	for (k = 0; k < src->nz; k++) {
-		dest->i[k] = src->i[k];
-		dest->data[k] = src->data[k];
-	}
+	for (k = 0; k < src->nnz; k++) dest->i[k] = src->i[k];
 	for (k = 0; k <= src->n; k++) dest->p[k] = src->p[k];
+	dcopy_ (&src->nnz, src->data, &ione, dest->data, &ione);
 
 	return dest;
 }
@@ -231,9 +244,8 @@ mm_real_copy_sparse (const mm_sparse *src)
 static mm_dense *
 mm_real_copy_dense (const mm_dense *src)
 {
-	mm_dense	*dest = mm_real_new (MM_REAL_DENSE, src->symm, src->m, src->n, src->nz);
-	dest->data = (double *) malloc (src->nz * sizeof (double));
-	dcopy_ (&src->nz, src->data, &ione, dest->data, &ione);
+	mm_dense	*dest = mm_real_new (MM_REAL_DENSE, src->symm, src->m, src->n, src->nnz);
+	dcopy_ (&src->nnz, src->data, &ione, dest->data, &ione);
 	return dest;
 }
 
@@ -253,11 +265,11 @@ mm_real_array_set_all (const int n, double *data, const double val)
 	return;
 }
 
-/*** set all data of mm to val ***/
+/*** set x->data to val ***/
 void
-mm_real_set_all (mm_real *mm, const double val)
+mm_real_set_all (mm_real *x, const double val)
 {
-	mm_real_array_set_all (mm->nz, mm->data, val);
+	mm_real_array_set_all (x->nnz, x->data, val);
 	return;
 }
 
@@ -268,37 +280,42 @@ mm_real_sparse_to_dense (const mm_sparse *s)
 	int			j;
 	mm_dense	*d;
 
-	if (!mm_real_is_sparse (s)) return false;
+	int			*si = s->i;
+	double		*sd = s->data;
+	double		*dd;
+
+	if (mm_real_is_dense (s)) return mm_real_copy (s);
 	d = mm_real_new (MM_REAL_DENSE, s->symm, s->m, s->n, s->m * s->n);
-	d->data = (double *) malloc (d->nz * sizeof (double));
 	mm_real_set_all (d, 0.);
 
+	dd = d->data;
 	for (j = 0; j < s->n; j++) {
-		int		k;
-		for (k = s->p[j]; k < s->p[j + 1]; k++) {
-			int		i = s->i[k];
-			d->data[i + j * s->m] = s->data[k];
-		}
+		int		k = s->p[j];
+		int		pend = s->p[j + 1];
+		for (; k < pend; k++) dd[si[k] + j * s->m] = sd[k];
 	}
 	return d;
 }
 
-/*** convert dense -> sparse ***/
-/* fabs (x->data[j]) < threshold are set to 0 */
+/*** convert dense -> sparse
+ * if fabs (x->data[j]) < threshold, set to 0 ***/
 mm_sparse *
 mm_real_dense_to_sparse (const mm_dense *d, const double threshold)
 {
 	int			j, k;
 	mm_sparse	*s;
 
-	if (!mm_real_is_dense (d)) return false;
-	s = mm_real_new (MM_REAL_SPARSE, d->symm, d->m, d->n, d->nz);
-	s->i = (int *) malloc (s->nz * sizeof (int));
-	s->p = (int *) malloc ((s->n + 1) * sizeof (int));
-	s->data = (double *) malloc (s->nz * sizeof (double));
+	int			*si;
+	double		*sd;
+	double		*dd = d->data;
+
+	if (mm_real_is_sparse (d)) return mm_real_copy (d);
+	s = mm_real_new (MM_REAL_SPARSE, d->symm, d->m, d->n, d->nnz);
+
+	si = s->i;
+	sd = s->data;
 
 	k = 0;
-	s->p[0] = 0;
 	for (j = 0; j < d->n; j++) {
 		int		i;
 		int		i0 = 0;
@@ -308,68 +325,88 @@ mm_real_dense_to_sparse (const mm_dense *d, const double threshold)
 			if (mm_real_is_upper (d)) i1 = j + 1;
 		}
 		for (i = i0; i < i1; i++) {
-			double	dij = d->data[i + j * d->m];
+			double	dij = dd[i + j * d->m];
 			if (fabs (dij) >= threshold) {
-				s->i[k] = i;
-				s->data[k] = dij;
+				si[k] = i;
+				sd[k] = dij;
 				k++;
 			}
 		}
 		s->p[j + 1] = k;
 	}
-	if (s->nz != k) mm_real_realloc (s, k);
+	if (s->nnz != k) mm_real_realloc (s, k);
 	return s;
 }
 
+/* find element that s->i[l] = j in the k-th column of s and return its index l */
+static int
+find_jth_row_element_of_sk (const int j, const mm_sparse *s, const int k)
+{
+	int		l = s->p[k];
+	int		pend = s->p[k + 1];
+	for (; l < pend; l++) {
+		int		si = s->i[l];
+		if (si < j) continue;
+		else if (si == j) return l;	// found
+		else break;
+	}
+	return -1;	// not found
+}
+
 /* convert sparse symmetric -> sparse general */
 static mm_sparse *
 mm_real_symmetric_to_general_sparse (const mm_sparse *x)
 {
 	int			j, m;
 	mm_sparse	*s;
-	if (!mm_real_is_symmetric (x)) return mm_real_copy (x);
 
-	s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, x->m, x->n, 2 * x->nz);
-	s->i = (int *) malloc (s->nz * sizeof (int));
-	s->data = (double *) malloc (s->nz * sizeof (double));
-	s->p = (int *) malloc ((s->n + 1) * sizeof (int));
+	int			*si;
+	double		*sd;
+	int			*xi;
+	double		*xd;
+
+	if (!mm_real_is_symmetric (x)) return mm_real_copy (x);
+	s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, x->m, x->n, 2 * x->nnz);
+
+	si = s->i;
+	sd = s->data;
+	xi = x->i;
+	xd = x->data;
 
 	m = 0;
-	s->p[0] = 0;
 	for (j = 0; j < x->n; j++) {
-		int		k, l;
+		int		k;
+		int		pend = x->p[j + 1];
 		if (mm_real_is_upper (x)) {
-			for (k = x->p[j]; k < x->p[j + 1]; k++) {
-				s->i[m] = x->i[k];
-				s->data[m++] = x->data[k];
+			for (k = x->p[j]; k < pend; k++) {
+				si[m] = xi[k];
+				sd[m++] = xd[k];
 			}
-			for (l = j + 1; l < x->n; l++) {
-				for (k = x->p[l]; k < x->p[l + 1]; k++) {
-					if (x->i[k] == j) {
-						s->i[m] = l;
-						s->data[m++] = x->data[k];
-						break;
-					}
+			for (k = j + 1; k < x->n; k++) {
+				int		l = find_jth_row_element_of_sk (j, x, k);
+				// if found
+				if (l >= 0) {
+					si[m] = k;
+					sd[m++] = xd[l];
 				}
 			}
 		} else if (mm_real_is_lower (x)) {
-			for (l = 0; l < j; l++) {
-				for (k = x->p[l]; k < x->p[l + 1]; k++) {
-					if (x->i[k] == j) {
-						s->i[m] = l;
-						s->data[m++] = x->data[k];
-						break;
-					}
+			for (k = 0; k < j; k++) {
+				int		l = find_jth_row_element_of_sk (j, x, k);
+				// if found
+				if (l >= 0) {
+					si[m] = k;
+					sd[m++] = xd[l];
 				}
 			}
-			for (k = x->p[j]; k < x->p[j + 1]; k++) {
-				s->i[m] = x->i[k];
-				s->data[m++] = x->data[k];
+			for (k = x->p[j]; k < pend; k++) {
+				si[m] = xi[k];
+				sd[m++] = xd[k];
 			}
 		}
 		s->p[j + 1] = m;
 	}
-	if (s->nz != m) mm_real_realloc (s, m);
+	if (s->nnz != m) mm_real_realloc (s, m);
 	return s;
 }
 
@@ -377,15 +414,17 @@ mm_real_symmetric_to_general_sparse (const mm_sparse *x)
 static mm_dense *
 mm_real_symmetric_to_general_dense (const mm_dense *x)
 {
-	int			i, j;
+	int			j;
 	mm_dense	*d = mm_real_copy (x);
 	if (!mm_real_is_symmetric (x)) return d;
 
 	for (j = 0; j < x->n; j++) {
 		if (mm_real_is_upper (x)) {
-			for (i = j + 1; i < x->m; i++) d->data[i + j * d->m] = x->data[j + i * x->m];
-		} else if (mm_real_is_lower (x)) {
-			for (i = 0; i < j; i++) d->data[i + j * d->m] = x->data[j + i * x->m];
+			int		i0 = j + 1;
+			int		len = x->m - i0;
+			dcopy_ (&len, x->data + j + i0 * x->m, &x->m, d->data + i0 + j * d->m, &ione);
+		} else if (j > 0 && mm_real_is_lower (x)) {
+			dcopy_ (&j, x->data + j, &x->m, d->data + j * d->m, &ione);
 		}
 	}
 	mm_real_set_general (d);
@@ -405,11 +444,6 @@ mm_real_seye (const int n)
 {
 	int			k;
 	mm_sparse	*s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, n, n, n);
-	s->i = (int *) malloc (n * sizeof (int));
-	s->data = (double *) malloc (n * sizeof (double));
-	s->p = (int *) malloc ((n + 1) * sizeof (int));
-
-	s->p[0] = 0;
 	for (k = 0; k < n; k++) {
 		s->i[k] = k;
 		s->data[k] = 1.;
@@ -424,7 +458,6 @@ mm_real_deye (const int n)
 {
 	int			k;
 	mm_dense	*d = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, n, n, n * n);
-	d->data = (double *) malloc (d->nz * sizeof (double));
 	mm_real_set_all (d, 0.);
 	for (k = 0; k < n; k++) d->data[k + k * n] = 1.;
 	return d;
@@ -434,28 +467,180 @@ mm_real_deye (const int n)
 mm_real *
 mm_real_eye (MMRealFormat format, const int n)
 {
-	if (n <= 0) error_and_exit ("mm_real_eye", "index out of range.", __FILE__, __LINE__);
+	if (n <= 0) error_and_exit ("mm_real_eye", "invalid size.", __FILE__, __LINE__);
 	return (format == MM_REAL_SPARSE) ? mm_real_seye (n) : mm_real_deye (n);
 }
 
+/* s = [s1; s2] */
+static mm_sparse *
+mm_real_vertcat_sparse (const mm_sparse *s1, const mm_sparse *s2)
+{
+	int			i, j, k;
+	int			m = s1->m + s2->m;
+	int			n = s1->n;
+	int			nnz = s1->nnz + s2->nnz;
+	mm_sparse	*s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, m, n, nnz);
+
+	k = 0;
+	for (j = 0; j < n; j++) {
+		int		len1 = s1->p[j + 1] - s1->p[j];
+		int		len2 = s2->p[j + 1] - s2->p[j];
+		int		pend;
+		dcopy_ (&len1, s1->data + s1->p[j], &ione, s->data + k, &ione);
+		pend = s1->p[j + 1];
+		for (i = s1->p[j]; i < pend; i++) s->i[k++] = s1->i[i];
+		dcopy_ (&len2, s2->data + s2->p[j], &ione, s->data + k, &ione);
+		pend = s2->p[j + 1];
+		for (i = s2->p[j]; i < pend; i++) s->i[k++] = s2->i[i] + s1->m;
+		s->p[j + 1] = k;
+	}
+	if (s->nnz != k) mm_real_realloc (s, k);
+	return s;
+}
+
+/* d = [d1; d2] */
+static mm_dense *
+mm_real_vertcat_dense (const mm_dense *d1, const mm_dense *d2)
+{
+	int			j;
+	int			m = d1->m + d2->m;
+	int			n = d1->n;
+	int			nnz = d1->nnz + d2->nnz;
+	mm_dense	*d = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, m, n, nnz);
+
+	for (j = 0; j < n; j++) {
+		dcopy_ (&d1->m, d1->data + j * d1->m, &ione, d->data + j * d->m, &ione);
+		dcopy_ (&d2->m, d2->data + j * d2->m, &ione, d->data + j * d->m + d1->m, &ione);
+	}
+	return d;
+}
+
+/*** x = [x1; x2] ***/
+mm_real *
+mm_real_vertcat (const mm_real *x1, const mm_real *x2)
+{
+	if ((mm_real_is_sparse (x1) && mm_real_is_dense (x1)) || (mm_real_is_dense (x1) && mm_real_is_sparse (x1)))
+		error_and_exit ("mm_real_vertcat", "format of matrix x1 and x2 are incompatible.", __FILE__, __LINE__);
+	if (mm_real_is_symmetric (x1) || mm_real_is_symmetric (x2))
+		error_and_exit ("mm_real_vertcat", "matrix must be general.", __FILE__, __LINE__);
+	if (x1->n != x2->n) error_and_exit ("mm_real_vertcat", "matrix size is incompatible.", __FILE__, __LINE__);
+
+	return (mm_real_is_sparse (x1)) ? mm_real_vertcat_sparse (x1, x2) : mm_real_vertcat_dense (x1, x2);
+}
+
+/* s = [s1, s2] */
+static mm_sparse *
+mm_real_holzcat_sparse (const mm_sparse *s1, const mm_sparse *s2)
+{
+	int			j, k;
+	int			m = s1->m;
+	int			n = s1->n + s2->n;
+	int			nnz = s1->nnz + s2->nnz;
+	mm_sparse	*s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, m, n, nnz);
+
+	for (k = 0; k < s1->nnz; k++) s->i[k] = s1->i[k];
+	for (k = 0; k < s2->nnz; k++) s->i[k + s1->nnz] = s2->i[k];
+	for (j = 0; j <= s1->n; j++) s->p[j] = s1->p[j];
+	for (j = 0; j <= s2->n; j++) s->p[j + s1->n] = s2->p[j] + s1->nnz;
+	dcopy_ (&s1->nnz, s1->data, &ione, s->data, &ione);
+	dcopy_ (&s2->nnz, s2->data, &ione, s->data + s1->nnz, &ione);
+
+	return s;
+}
+
+/* d = [d1, d2] */
+static mm_dense *
+mm_real_holzcat_dense (const mm_dense *d1, const mm_dense *d2)
+{
+	int			m = d1->m;
+	int			n = d1->n + d2->n;
+	int			nnz = d1->nnz + d2->nnz;
+	mm_dense	*d = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, m, n, nnz);
+
+	dcopy_ (&d1->nnz, d1->data, &ione, d->data, &ione);
+	dcopy_ (&d2->nnz, d2->data, &ione, d->data + d1->nnz, &ione);
+
+	return d;
+}
+
+/*** x = [x1, x2] ***/
+mm_real *
+mm_real_holzcat (const mm_real *x1, const mm_real *x2)
+{
+	if ((mm_real_is_sparse (x1) && mm_real_is_dense (x1)) || (mm_real_is_dense (x1) && mm_real_is_sparse (x1)))
+		error_and_exit ("mm_real_holzcat", "format of matrix x1 and x2 are incompatible.", __FILE__, __LINE__);
+	if (mm_real_is_symmetric (x1) || mm_real_is_symmetric (x2))
+		error_and_exit ("mm_real_holzcat", "matrix must be general.", __FILE__, __LINE__);
+	if (x1->m != x2->m) error_and_exit ("mm_real_holzcat", "matrix size is incompatible.", __FILE__, __LINE__);
+
+	return (mm_real_is_sparse (x1)) ? mm_real_holzcat_sparse (x1, x2) : mm_real_holzcat_dense (x1, x2);
+}
+
+/*** x(:,j) += alpha ***/
+void
+mm_real_xj_add_const (mm_real *x, const int j, const double alpha)
+{
+	int		k;
+	int		len;
+	double	*data;
+	if (mm_real_is_symmetric (x)) error_and_exit ("mm_real_xj_add_const", "matrix must be general.", __FILE__, __LINE__);
+	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_add_const", "index out of range.", __FILE__, __LINE__);
+
+	if (mm_real_is_sparse (x)) {
+		len = x->p[j + 1] - x->p[j];
+		data = x->data + x->p[j];
+	} else {
+		len = x->m;
+		data = x->data + j * x->m;
+	}
+	for (k = 0; k < len; k++) data[k] += alpha;
+
+	return;
+}
+
+/*** x(:,j) *= alpha ***/
+void
+mm_real_xj_scale (mm_real *x, const int j, const double alpha)
+{
+	int		len;
+	double	*data;
+	if (mm_real_is_symmetric (x)) error_and_exit ("mm_real_xj_scale", "matrix must be general.", __FILE__, __LINE__);
+	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_scale", "index out of range.", __FILE__, __LINE__);
+
+	if (mm_real_is_sparse (x)) {
+		len = x->p[j + 1] - x->p[j];
+		data = x->data + x->p[j];
+	} else {
+		len = x->m;
+		data = x->data + j * x->m;
+	}
+	dscal_ (&len, &alpha, data, &ione);
+
+	return;
+}
+
 /* sum |s(:,j)| */
 static double
 mm_real_sj_asum (const mm_sparse *s, const int j)
 {
 	int		size = s->p[j + 1] - s->p[j];
-	double	asum = dasum_ (&size, s->data + s->p[j], &ione);
+	double	*sd = s->data;
+	double	asum = dasum_ (&size, sd + s->p[j], &ione);
 	if (mm_real_is_symmetric (s)) {
-		int		l;
-		int		l0 = (mm_real_is_upper (s)) ? j + 1 : 0;
-		int		l1 = (mm_real_is_upper (s)) ? s->n : j;
-		for (l = l0; l < l1; l++) {
-			int		k;
-			for (k = s->p[l]; k < s->p[l + 1]; k++) {
-				if (s->i[k] == j) {
-					asum += fabs (s->data[k]);
-					break;
-				}
-			}
+		int		k;
+		int		k0;
+		int		k1;
+		if (mm_real_is_upper (s)) {
+			k0 = j + 1;
+			k1 = s->n;
+		} else {
+			k0 = 0;
+			k1 = j;
+		}
+		for (k = k0; k < k1; k++) {
+			int		l = find_jth_row_element_of_sk (j, s, k);
+			// if found
+			if (l >= 0) asum += fabs (sd[l]);
 		}
 	}
 	return asum;
@@ -496,20 +681,25 @@ mm_real_xj_asum (const mm_real *x, const int j)
 static double
 mm_real_sj_sum (const mm_sparse *s, const int j)
 {
-	int		k;
+	int		k = s->p[j];
+	int		pend = s->p[j + 1];
+	double	*sd = s->data;
 	double	sum = 0.;
-	for (k = s->p[j]; k < s->p[j + 1]; k++) sum += s->data[k];
+	for (; k < pend; k++) sum += sd[k];
 	if (mm_real_is_symmetric (s)) {
-		int		l;
-		int		l0 = (mm_real_is_upper (s)) ? j + 1 : 0;
-		int		l1 = (mm_real_is_upper (s)) ? s->n : j;
-		for (l = l0; l < l1; l++) {
-			for (k = s->p[l]; k < s->p[l + 1]; k++) {
-				if (s->i[k] == j) {
-					sum += s->data[k];
-					break;
-				}
-			}
+		int		k0;
+		int		k1;
+		if (mm_real_is_upper (s)) {
+			k0 = j + 1;
+			k1 = s->n;
+		} else {
+			k0 = 0;
+			k1 = j;
+		}
+		for (k = k0; k < k1; k++) {
+			int		l = find_jth_row_element_of_sk (j, s, k);
+			// if found
+			if (l >= 0) sum += sd[l];
 		}
 	}
 	return sum;
@@ -548,119 +738,138 @@ mm_real_xj_sum (const mm_real *x, const int j)
 	return (mm_real_is_sparse (x)) ? mm_real_sj_sum (x, j) : mm_real_dj_sum (x, j);
 }
 
-/* dot s(:,j) */
+/* sum_i s(i,j)^2 */
 static double
-mm_real_sj_dot (const mm_sparse *s, const int j)
+mm_real_sj_ssq (const mm_sparse *s, const int j)
 {
 	int		size = s->p[j + 1] - s->p[j];
-	double	dot = ddot_ (&size, s->data + s->p[j], &ione, s->data + s->p[j], &ione);
+	double	*sd = s->data;
+	double	ssq = ddot_ (&size, sd + s->p[j], &ione, sd + s->p[j], &ione);
 	if (mm_real_is_symmetric (s)) {
-		int		l;
-		int		l0 = (mm_real_is_upper (s)) ? j + 1 : 0;
-		int		l1 = (mm_real_is_upper (s)) ? s->n : j;
-		for (l = l0; l < l1; l++) {
-			int		k;
-			for (k = s->p[l]; k < s->p[l + 1]; k++) {
-				if (s->i[k] == j) {
-					dot += pow (s->data[k], 2.);
-					break;
-				}
-			}
+		int		k;
+		int		k0;
+		int		k1;
+		if (mm_real_is_upper (s)) {
+			k0 = j + 1;
+			k1 = s->n;
+		} else {
+			k0 = 0;
+			k1 = j;
+		}
+		for (k = k0; k < k1; k++) {
+			int		l = find_jth_row_element_of_sk (j, s, k);
+			// if found
+			if (l >= 0) ssq += pow (sd[l], 2.);
 		}
 	}
-	return dot;
+	return ssq;
 }
 
-/* norm2 d(:,j) */
+/* sum_i d(i,j)^2 */
 static double
-mm_real_dj_dot (const mm_dense *d, const int j)
+mm_real_dj_ssq (const mm_dense *d, const int j)
 {
-	double	dot;
-	if (!mm_real_is_symmetric (d)) dot = ddot_ (&d->m, d->data + j * d->m, &ione, d->data + j * d->m, &ione);
+	double	ssq;
+	if (!mm_real_is_symmetric (d)) ssq = ddot_ (&d->m, d->data + j * d->m, &ione, d->data + j * d->m, &ione);
 	else {
 		int		len;
-		dot = 0.;
+		ssq = 0.;
 		if (mm_real_is_upper (d)) {
 			len = j;
-			dot = ddot_ (&len, d->data + j * d->m, &ione, d->data + j * d->m, &ione);
+			ssq = ddot_ (&len, d->data + j * d->m, &ione, d->data + j * d->m, &ione);
 			len = d->m - j;
-			dot += ddot_ (&len, d->data + j * d->m + j, &d->m, d->data + j * d->m + j, &d->m);
+			ssq += ddot_ (&len, d->data + j * d->m + j, &d->m, d->data + j * d->m + j, &d->m);
 		} else if (mm_real_is_lower (d)) {
 			len = d->m - j;
-			dot = ddot_ (&len, d->data + j * d->m + j, &ione, d->data + j * d->m + j, &ione);
+			ssq = ddot_ (&len, d->data + j * d->m + j, &ione, d->data + j * d->m + j, &ione);
 			len = j;
-			dot += ddot_ (&len, d->data + j, &d->m, d->data + j, &d->m);
+			ssq += ddot_ (&len, d->data + j, &d->m, d->data + j, &d->m);
 		}
 	}
-	return dot;
+	return ssq;
 }
 
-/*** dot x(:,j) ***/
+/*** sum_i x(i,j)^2 ***/
 double
-mm_real_xj_dot (const mm_real *x, const int j)
+mm_real_xj_ssq (const mm_real *x, const int j)
 {
-	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_dot", "index out of range.", __FILE__, __LINE__);
-	return (mm_real_is_sparse (x)) ? mm_real_sj_dot (x, j) : mm_real_dj_dot (x, j);
+	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_ssq", "index out of range.", __FILE__, __LINE__);
+	return (mm_real_is_sparse (x)) ? mm_real_sj_ssq (x, j) : mm_real_dj_ssq (x, j);
 }
 
 /*** norm2 x(:,j) ***/
 double
 mm_real_xj_nrm2 (const mm_real *x, const int j)
 {
-	double	dot;
+	double	ssq;
 	if (j < 0 || x->n <= j) error_and_exit ("mm_real_xj_nrm2", "index out of range.", __FILE__, __LINE__);
-	dot = mm_real_xj_dot (x, j);
-	return sqrt (dot);
+	ssq = mm_real_xj_ssq (x, j);
+	return sqrt (ssq);
 }
 
-/* s * y, where s is sparse matrix and y is dense vector */
+/* alpha * s * y, where s is sparse matrix and y is dense vector */
 static mm_dense *
-mm_real_s_dot_y (bool trans, const double alpha, const mm_sparse *s, const mm_dense *y, const double beta)
+mm_real_s_dot_y (bool trans, const double alpha, const mm_sparse *s, const mm_dense *y)
 {
 	int			j;
 	int			m;
-	mm_dense	*d;
-	m = (trans) ? s->n : s->m;
-	
-	d = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, m, 1, m);
-	d->data = (double *) malloc (d->nz * sizeof (double));
-	mm_real_set_all (d, beta);
+	mm_dense	*c;
 
+	double		*sd = s->data;
+	double		*yd = y->data;
+	double		*cd;
+
+	m = (trans) ? s->n : s->m;
+
+	c = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, m, 1, m);
+	mm_real_set_all (c, 0.);
+
+	cd = c->data;
 	for (j = 0; j < s->n; j++) {
-		int		k;
-		for (k = s->p[j]; k < s->p[j + 1]; k++) {
-			int		si = (trans) ? j : s->i[k];
-			int		sj = (trans) ? s->i[k] : j;
-			d->data[si] += s->data[k] * y->data[sj];
-			if (mm_real_is_symmetric (s) && j != s->i[k]) d->data[sj] += s->data[k] * y->data[si];
+		int		k = s->p[j];
+		int		pend = s->p[j + 1];
+		for (; k < pend; k++) {
+			int		si;
+			int		sj;
+			if (trans) {
+				si = j;
+				sj = s->i[k];
+			} else {
+				si = s->i[k];
+				sj = j;
+			}
+			cd[si] += alpha * sd[k] * yd[sj];
+			if (mm_real_is_symmetric (s) && j != s->i[k]) cd[sj] += alpha * sd[k] * yd[si];
 		}		
 	}
-	return d;
+	return c;
 }
 
-/* d * y, where d is dense matrix and y is dense vector */
+/* alpha * d * y, where d is dense matrix and y is dense vector */
 static mm_dense *
-mm_real_d_dot_y (bool trans, const double alpha, const mm_dense *d, const mm_dense *y, const double beta)
+mm_real_d_dot_y (bool trans, const double alpha, const mm_dense *d, const mm_dense *y)
 {
 	int			m;
 	mm_dense	*c;
 	m = (trans) ? d->n : d->m;
 
 	c = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, m, 1, m);
-	c->data = (double *) malloc (c->nz * sizeof (double));
+	mm_real_set_all (c, 0.);
 
-	if (!mm_real_is_symmetric (d))
-		dgemv_ ((trans) ? "T" : "N", &d->m, &d->n, &alpha, d->data, &d->m, y->data, &ione, &beta, c->data, &ione);
-	else {
+	if (!mm_real_is_symmetric (d)) {
+		// c = alpha*d*y + 0*c
+		dgemv_ ((trans) ? "T" : "N", &d->m, &d->n, &alpha, d->data, &d->m, y->data, &ione, &dzero, c->data, &ione);
+	} else {
 		char	uplo = (mm_real_is_upper (d)) ? 'U' : 'L';
-		dsymv_ (&uplo, &d->m, &alpha, d->data, &d->m, y->data, &ione, &beta, c->data, &ione);
+		// c = alpha*d*y + 0*c
+		dsymv_ (&uplo, &d->m, &alpha, d->data, &d->m, y->data, &ione, &dzero, c->data, &ione);
 	}
 	return c;
 }
 
-/*** x * y, where x is matrix and y is dense vector ***/
+/*** alpha * x * y, where x is sparse/dense matrix and y is dense vector ***/
 mm_dense *
-mm_real_x_dot_y (bool trans, const double alpha, const mm_real *x, const mm_dense *y, const double beta)
+mm_real_x_dot_y (bool trans, const double alpha, const mm_real *x, const mm_dense *y)
 {
 	if (!mm_real_is_dense (y)) error_and_exit ("mm_real_x_dot_y", "y must be dense.", __FILE__, __LINE__);
 	if (mm_real_is_symmetric (y)) error_and_exit ("mm_real_x_dot_y", "y must be general.", __FILE__, __LINE__);
@@ -668,27 +877,36 @@ mm_real_x_dot_y (bool trans, const double alpha, const mm_real *x, const mm_dens
 	if ((trans && x->m != y->m) || (!trans && x->n != y->m))
 		error_and_exit ("mm_real_x_dot_y", "vector and matrix dimensions do not match.", __FILE__, __LINE__);
 
-	return (mm_real_is_sparse (x)) ? mm_real_s_dot_y (trans, alpha, x, y, beta) : mm_real_d_dot_y (trans, alpha, x, y, beta);
+	return (mm_real_is_sparse (x)) ? mm_real_s_dot_y (trans, alpha, x, y) : mm_real_d_dot_y (trans, alpha, x, y);
 }
 
 /* s(:,j)' * y */
 static double
 mm_real_sj_trans_dot_y (const mm_sparse *s, const int j, const mm_dense *y)
 {
-	int	k;
 	double	val = 0;
-	for (k = s->p[j]; k < s->p[j + 1]; k++) val += s->data[k] * y->data[s->i[k]];
+
+	int		*si = s->i;
+	double	*sd = s->data;
+	double	*yd = y->data;
+
+	int		k = s->p[j];
+	int		pend = s->p[j + 1];
+	for (; k < pend; k++) val += sd[k] * yd[si[k]];
 	if (mm_real_is_symmetric (s)) {
-		int		l;
-		int		l0 = (mm_real_is_upper (s)) ? j + 1 : 0;
-		int		l1 = (mm_real_is_upper (s)) ? s->n : j;
-		for (l = l0; l < l1; l++) {
-			for (k = s->p[l]; k < s->p[l + 1]; k++) {
-				if (s->i[k] == j) {
-					val += s->data[k] * y->data[l];
-					break;
-				}
-			}
+		int		k0;
+		int		k1;
+		if (mm_real_is_upper (s)) {
+			k0 = j + 1;
+			k1 = s->n;
+		} else {
+			k0 = 0;
+			k1 = j;
+		}
+		for (k = k0; k < k1; k++) {
+			int		l = find_jth_row_element_of_sk (j, s, k);
+			// if found
+			if (l >= 0) val += sd[l] * yd[k];
 		}
 	}
 	return val;
@@ -734,19 +952,27 @@ mm_real_xj_trans_dot_y (const mm_real *x, const int j, const mm_dense *y)
 static void
 mm_real_asjpy (const double alpha, const mm_sparse *s, const int j, mm_dense *y)
 {
-	int		k;
-	for (k = s->p[j]; k < s->p[j + 1]; k++) y->data[s->i[k]] += alpha * s->data[k];
+	int		*si = s->i;
+	double	*sd = s->data;
+	double	*yd = y->data;
+
+	int		k = s->p[j];
+	int		pend = s->p[j + 1];
+	for (; k < pend; k++) yd[si[k]] += alpha * sd[k];
 	if (mm_real_is_symmetric (s)) {
-		int		l;
-		int		l0 = (mm_real_is_upper (s)) ? j + 1 : 0;
-		int		l1 = (mm_real_is_upper (s)) ? s->n : j;
-		for (l = l0; l < l1; l++) {
-			for (k = s->p[l]; k < s->p[l + 1]; k++) {
-				if (s->i[k] == j) {
-					y->data[l] += alpha * s->data[k];
-					break;
-				}
-			}
+		int		k0;
+		int		k1;
+		if (mm_real_is_upper (s)) {
+			k0 = j + 1;
+			k1 = s->n;
+		} else {
+			k0 = 0;
+			k1 = j;
+		}
+		for (k = k0; k < k1; k++) {
+			int		l = find_jth_row_element_of_sk (j, s, k);
+			// if found
+			if (l >= 0) yd[k] += alpha * sd[l];
 		}
 	}
 	return;
@@ -791,19 +1017,27 @@ mm_real_axjpy (const double alpha, const mm_real *x, const int j, mm_dense *y)
 static void
 mm_real_asjpy_atomic (const double alpha, const mm_sparse *s, const int j, mm_dense *y)
 {
-	int		k;
-	for (k = s->p[j]; k < s->p[j + 1]; k++) atomic_add (y->data + s->i[k], alpha * s->data[k]);
+	int		*si = s->i;
+	double	*sd = s->data;
+	double	*yd = y->data;
+
+	int		k = s->p[j];
+	int		pend = s->p[j + 1];
+	for (; k < pend; k++) atomic_add (yd + si[k], alpha * sd[k]);
 	if (mm_real_is_symmetric (s)) {
-		int		l;
-		int		l0 = (mm_real_is_upper (s)) ? j + 1 : 0;
-		int		l1 = (mm_real_is_upper (s)) ? s->n : j;
-		for (l = l0; l < l1; l++) {
-			for (k = s->p[l]; k < s->p[l + 1]; k++) {
-				if (s->i[k] == j) {
-					atomic_add (y->data + l, alpha * s->data[k]);
-					break;
-				}
-			}
+		int		k0;
+		int		k1;
+		if (mm_real_is_upper (s)) {
+			k0 = j + 1;
+			k1 = s->n;
+		} else {
+			k0 = 0;
+			k1 = j;
+		}
+		for (k = k0; k < k1; k++) {
+			int		l = find_jth_row_element_of_sk (j, s, k);
+			// if found
+			if (l >= 0) atomic_add (yd + k, alpha * sd[l]);
 		}
 	}
 	return;
@@ -813,27 +1047,24 @@ mm_real_asjpy_atomic (const double alpha, const mm_sparse *s, const int j, mm_de
 static void
 mm_real_adjpy_atomic (const double alpha, const mm_dense *d, const int j, mm_dense *y)
 {
+	double	*dd = d->data;
+	double	*yd = y->data;
+
 	int		k;
-	double	*dj;
 	if (!mm_real_is_symmetric (d)) {
-		dj = d->data + j * d->m;
-		for (k = 0; k < d->m; k++) atomic_add (y->data + k, alpha * dj[k]);
+		for (k = 0; k < d->m; k++) atomic_add (yd + k, alpha * dd[j * d->m + k]);
 	} else {
 		int		len;
 		if (mm_real_is_upper (d)) {
 			len = j;
-			dj = d->data + j * d->m;
-			for (k = 0; k < len; k++) atomic_add (y->data + k, alpha * dj[k]);
+			for (k = 0; k < len; k++) atomic_add (yd + k, alpha * dd[j * d->m + k]);
 			len = d->m - j;
-			dj = d->data + j * d->m + j;
-			for (k = 0; k < len; k++) atomic_add (y->data + j + k, alpha * dj[k * d->m]);
+			for (k = 0; k < len; k++) atomic_add (yd + j + k, alpha * dd[j * d->m + j + k * d->m]);
 		} else if (mm_real_is_lower (d)) {
 			len = d->m - j;
-			dj = d->data + j * d->m + j;
-			for (k = 0; k < len; k++) atomic_add (y->data + j + k, alpha * dj[k]);
+			for (k = 0; k < len; k++) atomic_add (yd + j + k, alpha * dd[j * d->m + j + k]);
 			len = j;
-			dj = d->data + j;
-			for (k = 0; k < len; k++) atomic_add (y->data + k, alpha * dj[k * d->m]);
+			for (k = 0; k < len; k++) atomic_add (yd + k, alpha * dd[j + k * d->m]);
 		}
 	}
 	return;
@@ -857,21 +1088,22 @@ static mm_sparse *
 mm_real_fread_sparse (FILE *fp, MM_typecode typecode)
 {
 	int			k, l;
-	int			m, n, nz;
+	int			m, n, nnz;
 	int			*j;
 	mm_sparse	*s;
 
-	mm_read_mtx_crd_size (fp, &m, &n, &nz);
-	s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, m, n, nz);
-	s->i = (int *) malloc (s->nz * sizeof (int));
-	s->data = (double *) malloc (s->nz * sizeof (double));
-	s->p = (int *) malloc ((s->n + 1) * sizeof (int));
+	if (mm_read_mtx_crd_size (fp, &m, &n, &nnz) != 0) return NULL;
+	s = mm_real_new (MM_REAL_SPARSE, MM_REAL_GENERAL, m, n, nnz);
 
-	j = (int *) malloc (s->nz * sizeof (int));
-	mm_read_mtx_crd_data (fp, s->m, s->n, s->nz, s->i, j, s->data, typecode);
+	j = (int *) malloc (s->nnz * sizeof (int));
+	if (mm_read_mtx_crd_data (fp, s->m, s->n, s->nnz, s->i, j, s->data, typecode) != 0) {
+		free (j);
+		mm_real_free (s);
+		return NULL;
+	}
 
 	l = 0;
-	for (k = 0; k < s->nz; k++) {
+	for (k = 0; k < s->nnz; k++) {
 		s->i[k]--;	// fortran -> c
 		while (l < j[k]) s->p[l++] = k;
 	}
@@ -879,7 +1111,7 @@ mm_real_fread_sparse (FILE *fp, MM_typecode typecode)
 
 	if (mm_is_symmetric (typecode)) {
 		mm_real_set_symmetric (s);
-		for (k = 0; k < s->nz; k++) {
+		for (k = 0; k < s->nnz; k++) {
 			if (s->i[k] == j[k] - 1) continue;
 			(s->i[k] < j[k] - 1) ? mm_real_set_upper (s) : mm_real_set_lower (s);
 			break;
@@ -899,14 +1131,13 @@ mm_real_fread_dense (FILE *fp, MM_typecode typecode)
 	int			ret;
 	mm_dense	*d;
 
-	mm_read_mtx_array_size (fp, &m, &n);
+	if (mm_read_mtx_array_size (fp, &m, &n) != 0) return NULL;
 	d = mm_real_new (MM_REAL_DENSE, MM_REAL_GENERAL, m, n, m * n);
-	d->data = (double *) malloc (d->nz * sizeof (double));
 
 	k = 0;
 	do {
 		ret = fscanf (fp, "%lf", &d->data[k]);
-		if (ret > 0 && ++k >= d->nz) break;
+		if (ret > 0 && ++k >= d->nnz) break;
 	} while (ret != EOF);
 
 	return d;
@@ -918,29 +1149,28 @@ mm_real_fread (FILE *fp)
 {
 	MM_typecode	typecode;
 	mm_real		*x;
-	mm_read_banner (fp, &typecode);
+	if (mm_read_banner (fp, &typecode) != 0) error_and_exit ("mm_real_fread", "failed to read mm_real.", __FILE__, __LINE__);
 	if (!is_type_supported (typecode)) {
 		char	msg[128];
 		sprintf (msg, "matrix type does not supported :[%s].", mm_typecode_to_str (typecode));
 		error_and_exit ("mm_real_fread", msg, __FILE__, __LINE__);
 	}
 	x = (mm_is_sparse (typecode)) ? mm_real_fread_sparse (fp, typecode) : mm_real_fread_dense (fp, typecode);
-	if (mm_real_is_symmetric (x) && x->m != x->n) {
-		mm_real_free (x);
-		error_and_exit ("mm_real_fread", "symmetric matrix must be square.", __FILE__, __LINE__);
-	}
+	if (mm_real_is_symmetric (x) && x->m != x->n) error_and_exit ("mm_real_fread", "symmetric matrix must be square.", __FILE__, __LINE__);
 	return x;
 }
 
 /* fwrite sparse */
 static void
-mm_real_fwrite_sparse (FILE *stream, mm_sparse *s, const char *format)
+mm_real_fwrite_sparse (FILE *stream, const mm_sparse *s, const char *format)
 {
-	int		j, k;
+	int		j;
 	mm_write_banner (stream, s->typecode);
-	mm_write_mtx_crd_size (stream, s->m, s->n, s->nz);
+	mm_write_mtx_crd_size (stream, s->m, s->n, s->nnz);
 	for (j = 0; j < s->n; j++) {
-		for (k = s->p[j]; k < s->p[j + 1]; k++) {
+		int		k = s->p[j];
+		int		pend = s->p[j + 1];
+		for (; k < pend; k++) {
 			fprintf (stream, "%d %d ", s->i[k] + 1, j + 1);	// c -> fortran
 			fprintf (stream, format, s->data[k]);
 			fprintf (stream, "\n");
@@ -951,12 +1181,12 @@ mm_real_fwrite_sparse (FILE *stream, mm_sparse *s, const char *format)
 
 /* fwrite dense */
 static void
-mm_real_fwrite_dense (FILE *stream, mm_dense *d, const char *format)
+mm_real_fwrite_dense (FILE *stream, const mm_dense *d, const char *format)
 {
 	int		k;
 	mm_write_banner (stream, d->typecode);
 	mm_write_mtx_array_size (stream, d->m, d->n);
-	for (k = 0; k < d->nz; k++) {
+	for (k = 0; k < d->nnz; k++) {
 		fprintf (stream, format, d->data[k]);
 		fprintf (stream, "\n");
 	}
@@ -965,7 +1195,7 @@ mm_real_fwrite_dense (FILE *stream, mm_dense *d, const char *format)
 
 /*** fwrite in MatrixMarket format ***/
 void
-mm_real_fwrite (FILE *stream, mm_real *x, const char *format)
+mm_real_fwrite (FILE *stream, const mm_real *x, const char *format)
 {
 	return (mm_real_is_sparse (x)) ? mm_real_fwrite_sparse (stream, x, format) : mm_real_fwrite_dense (stream, x, format);
 }
